@@ -35,7 +35,7 @@ from functools import wraps
 from datetime import datetime
 from collections import Iterable
 
-version = '0.8.1'
+version = '0.9.0'
 
 
 def _get_file_name(func):
@@ -239,9 +239,11 @@ class that(object):
             self._src(*self._callable_args, **self._callable_kw)
 
         except Exception, e:
+            err = str(e)
+            exc = type(e)
+
             if isinstance(exc, basestring):
                 msg = exc
-                exc = type(e)
 
             if isinstance(exc, type) and issubclass(exc, Exception):
                 if not isinstance(e, exc):
@@ -249,11 +251,18 @@ class that(object):
                         '%r should raise %r, but raised %r' % (
                             self._src, exc, e.__class__))
 
-                if isinstance(msg, basestring) and msg not in unicode(e):
-                    raise AssertionError('%r raised %s, but the exception message does not match.\n\nEXPECTED:\n%r\n\nGOT:\n%r' % (self._src, type(e).__name__, msg, str(e)))
+                if isinstance(msg, basestring) and msg not in err:
+                    raise AssertionError('''
+                    %r raised %s, but the exception message does not
+                    match.\n\nEXPECTED:\n%s\n\nGOT:\n%s'''.strip() % (
+                            self._src,
+                            type(e).__name__,
+                            msg, err))
 
-            elif isinstance(msg, basestring) and msg not in unicode(e):
-                raise AssertionError('When calling %r the exception message does not match. Expected: %r\n got:\n %r' % (self._src, msg, str(e)))
+            elif isinstance(msg, basestring) and msg not in err:
+                raise AssertionError(
+                    'When calling %r the exception message does not match. ' \
+                    'Expected: %r\n got:\n %r' % (self._src, msg, err))
 
             else:
                 raise e
@@ -277,6 +286,14 @@ class that(object):
 
         return True
 
+    def deep_equals(self, dst):
+        deep = DeepComparison(self._src, dst)
+        comparison = deep.compare()
+        if isinstance(comparison, bool):
+            return comparison
+
+        raise comparison.as_assertion(), deep.explanation
+
     def equals(self, dst):
         if self._attribute and is_iterable(self._src):
             msg = '%r[%d].%s should be %r, but is %r'
@@ -709,3 +726,147 @@ def action_for(context, provides=None, depends_on=None):
         return wrapper
 
     return decorate_and_absorb
+
+
+class DeepExplanation(unicode):
+    def get_header(self, X, Y, suffix):
+        return """given
+X = %r
+    and
+Y = %r
+%s""".strip() % (X, Y, suffix)
+
+    def as_assertion(self, X, Y):
+        raise AssertionError(self.get_header(X, Y, self))
+
+
+class DeepComparison(object):
+    def __init__(self, X, Y, level=0, parent=None):
+        self.operands = X, Y
+        self.level = level
+        self.parent = parent
+        self._context = None
+
+    def is_simple(self, obj):
+        return isinstance(obj, (
+            int, long, float, basestring,
+        ))
+
+    def compare_complex_stuff(self, X, Y):
+        kind = type(X)
+        mapping = {
+            dict: self.compare_dicts,
+            list: self.compare_iterables,
+            tuple: self.compare_iterables,
+        }
+        return mapping.get(kind, self.compare_generic)(X, Y)
+
+    def compare_generic(self, X, Y):
+        type_X, type_Y = map(type, (X, Y))
+        if type_X != type_Y:
+            m = 'X has type %s whereas Y has type %s' % (type_X, type_Y)
+            return DeepExplanation(m)
+
+        elif X == Y:
+            return True
+        else:
+            c = self.get_context()
+            m = 'X{0} != Y{1}'.format(c.current_X_keys, c.current_Y_keys)
+            return DeepExplanation(m)
+
+    def compare_dicts(self, X, Y):
+        x_keys = X.keys()
+        y_keys = Y.keys()
+
+        diff_x = list(set(x_keys).difference(set(y_keys)))
+        diff_y = list(set(y_keys).difference(set(x_keys)))
+        if diff_x:
+            msg = "X has the key \"%s\" whereas Y doesn't" % diff_x[0]
+            return DeepExplanation(msg)
+        elif diff_y:
+            msg = "Y has the key \"%s\" whereas X doesn't" % diff_y[0]
+            return DeepExplanation(msg)
+        elif X == Y:
+            return True
+        else:
+            for key_X, key_Y in zip(x_keys, y_keys):
+                self.key_X = key_X
+                self.key_Y = key_Y
+                value_X = X[key_X]
+                value_Y = Y[key_Y]
+                child = DeepComparison(
+                    value_X,
+                    value_Y,
+                    level=self.level + 1,
+                    parent=self,
+                ).compare()
+                if child:
+                    return child
+
+    def get_context(self):
+        if self._context:
+            return self._context
+
+        X_keys = []
+        Y_keys = []
+
+        comp = self
+        while comp.parent:
+            X_keys.insert(0, comp.parent.key_X)
+            Y_keys.insert(0, comp.parent.key_Y)
+            comp = comp.parent
+
+        class ComparisonContext:
+            current_X_keys = '[%s]' % ']['.join(map(repr, X_keys))
+            current_Y_keys = '[%s]' % ']['.join(map(repr, Y_keys))
+            parent = comp
+
+        self._context = ComparisonContext()
+        return self._context
+
+    def compare_iterables(self, X, Y):
+        len_X, len_Y = map(len, (X, Y))
+        if len_X > len_Y:
+            msg = "X has %d items whereas Y has only %d" % (len_X, len_Y)
+            return DeepExplanation(msg)
+        elif len_X < len_Y:
+            msg = "Y has %d items whereas X has only %d" % (len_Y, len_X)
+            return DeepExplanation(msg)
+        elif X == Y:
+            return True
+        else:
+            for i, (value_X, value_Y) in enumerate(zip(X, Y)):
+                self.key_X = self.key_Y = i
+                if value_Y != value_X:
+                    m = 'Y[{0}] is %r whereas should be like %r as in X[{0}]'
+                    msg = m.format(i) % (value_Y, value_X)
+                    return DeepExplanation(msg)
+
+    def compare(self):
+        X, Y = self.operands
+
+        if self.is_simple(X) and self.is_simple(Y):  # both simple
+            if X == Y:
+                return True
+            c = self.get_context()
+            m = "X{0} is %r whereas Y{1} is %r"
+            msg = m.format(c.current_X_keys, c.current_Y_keys) % (X, Y)
+            return DeepExplanation(msg)
+
+        elif type(X) is not type(Y):  # different types
+            xname, yname = map(lambda _: type(_).__name__, (X, Y))
+            msg = 'X is a %s and Y is a %s instead' % (xname, yname)
+            exp = DeepExplanation(msg)
+
+        else:
+            exp = self.compare_complex_stuff(X, Y)
+
+        if isinstance(exp, DeepExplanation):
+            c = self.get_context()
+            original_X, original_Y = c.parent.operands
+            raise exp.as_assertion(original_X, original_Y)
+
+        return exp
+
+    def explanation(self):
+        return self._explanation
